﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

namespace Pippin{
	public class FadeImage : MonoBehaviour {

        public static FadeImage Instance;

		[SerializeField] Image imagem;
		[SerializeField] SpriteRenderer sprite;

		[SerializeField] float minimo = .0f;
		[SerializeField] float maximo = 1f;
		[SerializeField] float duracao = 2.0f;

		float startTime;
		float finishTime;
		bool madeFadeIn;
		bool fadeOparation;

        void Awake()
        {
            Instance = this;
        }

        void Start () {
			definirTempo ();
		}

		void Update () {
			if (fadeOparation) {
				if (madeFadeIn)
					realizarFadeIn ();
				else
					realizarFadeOut ();
			}
		}		

        /// <summary>
        /// Realizars the fade in.
        /// </summary>
		void realizarFadeIn() {

			float t = (Time.time - startTime) / duracao;

			if (imagem != null) {

				imagem.color = new Color (
					imagem.color.r, 
					imagem.color.g, 
					imagem.color.b,
					Mathf.SmoothStep(minimo, maximo, t)
				);

			} else {
				sprite.color = new Color (
					sprite.color.r, 
					sprite.color.g, 
					sprite.color.b,
					Mathf.SmoothStep(minimo, maximo, t)
				);
			}
		}

        /// <summary>
        /// Realizars the fade out.
        /// </summary>
		void realizarFadeOut() {
			float t = (Time.time - startTime) / duracao;

			if (imagem != null) {

				imagem.color = new Color (
					imagem.color.r, 
					imagem.color.g, 
					imagem.color.b,
					Mathf.SmoothStep(maximo, minimo, t)
				);

			} else {
				sprite.color = new Color (
					sprite.color.r, 
					sprite.color.g, 
					sprite.color.b,
					Mathf.SmoothStep(maximo, minimo, t)
				);
			}
		}

        /// <summary>
        /// Definirs the tempo.
        /// </summary>
		void definirTempo(){
			startTime = Time.time;
		}

        /// <summary>
        /// Finalizars the operacao fade.
        /// </summary>
        IEnumerator finalizarOperacaoFade(System.Action callback){
            yield return new WaitForSeconds(this.duracao);
			fadeOparation = false;

            if(madeFadeIn)
            {

                if(imagem != null)
                {
                    imagem.color = new Color(
                        imagem.color.r,
                        imagem.color.g,
                        imagem.color.b,
                        maximo
                    );
                }
                else
                {
                    sprite.color = new Color(
                        sprite.color.r,
                        sprite.color.g,
                        sprite.color.b,
                        maximo
                    );
                }

            }
            else
            {
                if (imagem != null)
                {
                    imagem.color = new Color(
                        imagem.color.r,
                        imagem.color.g,
                        imagem.color.b,
                        minimo
                    );
                }
                else
                {
                    sprite.color = new Color(
                        sprite.color.r,
                        sprite.color.g,
                        sprite.color.b,
                        minimo
                    );
                }
            }

            if (callback != null)
                callback();
        }

        /// <summary>
        /// Realiza o processo de fade com o tempo definido no metodo
        /// </summary>
        /// <param name="duracao">Tempo em que o script realiza o processo</param>
        /// <param name="fade">True para fadein, false para fadeout</param>
        public void _fade(float duracao, bool fade, System.Action callback = null){
			if (duracao != 0) 
				this.duracao = duracao;

			definirTempo ();

			madeFadeIn = fade;
			fadeOparation = true;
			CancelInvoke ();

            StartCoroutine(finalizarOperacaoFade(callback));
            StartCoroutine(finalizarOperacaoFade(callback));
        }

        /// <summary>
        /// Realiza o processo de fade com o tempo padrão definido no componente
        /// </summary>
        /// <param name="fade">Passar true para realizar o fadeIn, false para fadeout</param>
		public void _fade(bool fade){
			definirTempo ();

			madeFadeIn = fade;
			fadeOparation = true;
			CancelInvoke ();
            StartCoroutine(finalizarOperacaoFade(null));
        }
	}

}