﻿using System.Collections;
using System.Collections.Generic;
using Pippin.Player;
using UnityEngine;

namespace Pippin.Enemy
{
    public enum DirecaoEnemy
    {
        NONE, UP, RIGHT, DOWN, LEFT
    }

    [System.Serializable]
    public class AudioEnemy
    {
        public AudioClip[] caminhada;
        public AudioClip[] ataque;
        public AudioClip[] levandoDano;
        public AudioClip[] morrendo;

        AudioSource Fonte;

        bool esperarAudio;

        public void Init(AudioSource fonte)
        {
            Fonte = fonte;
        }

        public void Caminhada()
        {
            if(!esperarAudio && caminhada.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = caminhada[Random.Range(0, caminhada.Length-1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, ()=>{ esperarAudio = false; });
            }
        }

        public void Ataque()
        {
            if (!esperarAudio && ataque.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = ataque[Random.Range(0, ataque.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        public void LevandoDano()
        {
            if (!esperarAudio && levandoDano.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = levandoDano[Random.Range(0, levandoDano.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        public void Morrendo()
        {
            if (!esperarAudio && morrendo.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = morrendo[Random.Range(0, morrendo.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        IEnumerator EsperarAudioFinalizar(float time, System.Action callback)
        {
            yield return new WaitForSeconds(time);
            callback();
        }

    }

    [System.Serializable]
    public class AnimacaoEnemy
    {
        [Header("Clips de Animacoes")]
        public AnimationClip Ataque;
        public AnimationClip Caminhada;
        public AnimationClip damage;
        public AnimationClip morte;

        [Header("Ajustes de velocidade Animacoes")]
        public float multiplicadorWalk = 1f;
        public float multiplicadorIdle = 1f;
        public float multiplicadorAtaque = 1f;

        Animator _animador;
        Animator _parent;
        int ultimaDirecao;
        EnemyController _enemy;

        public void Init(Animator animador, Animator parent, EnemyController enemy)
        {
            _animador = animador;
            _parent = parent;
            _enemy = enemy;
            _animador.SetFloat("mul-walk", multiplicadorWalk);
            _animador.SetFloat("mul-ataque", multiplicadorAtaque);
            _animador.SetFloat("mul-idle", multiplicadorIdle);
        }

        public void TomarDano()
        {
            _parent.SetTrigger("dano");
            //_animador.SetTrigger("dano");
            _enemy._audio.LevandoDano();
        }

        public void Caminhar(DirecaoEnemy caminhada)
        {
            switch (caminhada)
            {
                case DirecaoEnemy.UP:
                    _animador.SetInteger("caminhar", 1);
                    ultimaDirecao = 1;
                    break;
                case DirecaoEnemy.RIGHT:
                    _animador.SetInteger("caminhar", 2);
                    ultimaDirecao = 2;
                    break;
                case DirecaoEnemy.DOWN:
                    _animador.SetInteger("caminhar", 3);
                    ultimaDirecao = 3;
                    break;
                case DirecaoEnemy.LEFT:
                    _animador.SetInteger("caminhar", 4);
                    ultimaDirecao = 4;
                    break;
                default:
                    _animador.SetInteger("caminhar", 0);
                    break;
            }

            _enemy._audio.Caminhada();
        }

        public void atacar(DirecaoEnemy direcao, System.Action esperarAtaqueFinalizar)
        {
            switch (direcao)
            {
                case DirecaoEnemy.UP:

                    _animador.SetInteger("atacar", 1);
                    _enemy.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () => {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
                case DirecaoEnemy.RIGHT:

                    _animador.SetInteger("atacar", 2);
                    _enemy.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () => {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
                case DirecaoEnemy.DOWN:

                    _animador.SetInteger("atacar", 3);
                    _enemy.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () => {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
                case DirecaoEnemy.LEFT:

                    _animador.SetInteger("atacar", 4);
                    _enemy.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () => {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
            }

            _enemy._audio.Ataque();
        }

        public void Morrer(System.Action esperarAnimacao = null)
        {
            ResetarParametrosAnimator();
            switch (Direcao)
            {
                case DirecaoEnemy.UP:

                    _animador.SetInteger("morto", 1);
                    if (esperarAnimacao != null)
                        _enemy.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
                case DirecaoEnemy.RIGHT:

                    _animador.SetInteger("morto", 2);
                    if (esperarAnimacao != null)
                        _enemy.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
                case DirecaoEnemy.DOWN:

                    _animador.SetInteger("morto", 3);
                    if (esperarAnimacao != null)
                        _enemy.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
                case DirecaoEnemy.LEFT:

                    _animador.SetInteger("morto", 4);
                    if (esperarAnimacao != null)
                        _enemy.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
            }

            _enemy._audio.Morrendo();
        }

        IEnumerator esperarAcaoAnimacao(float time, System.Action callback)
        {
            yield return new WaitForSeconds(time);
            callback();
        }

        public void ResetarParametrosAnimator()
        {
            _animador.SetInteger("caminhar", 0);
            _animador.SetInteger("atacar", 0);
            _animador.Rebind();
        }

        public bool Atacando { get => _animador.GetInteger("atacar") != 0; }

        public bool Pescando { get => _animador.GetBool("pescando"); }

        public DirecaoEnemy Direcao { get => (DirecaoEnemy)ultimaDirecao; }
    }

    [System.Serializable]
    public class AtaqueEnemy
    {
        public int valorDano;
        public float delayAtaque = 0.2f;
        public float frequenciaAtaque = 2f;
        public bool empurrarPlayer;
        [HideInInspector]
        public bool PlayerAreaAtaque;

        AnimacaoEnemy _animacao;
        EnemyController _enemy;
        EnemyAreaAtaque _area;

        bool iniciouProcessoAtaque;

        public void Init(AnimacaoEnemy animacao, EnemyController enemy, EnemyAreaAtaque area)
        {
            _animacao = animacao;
            _enemy = enemy;
            _area = area;
            _area.ataque = this;
        }

        public void Update()
        {
            if (_animacao.Atacando || iniciouProcessoAtaque)
                return;

            if(PlayerAreaAtaque)
            {
                iniciouProcessoAtaque = true;
                _enemy.IniciarCorotina(DelayAtaque());
            }
        }

        IEnumerator DelayAtaque()
        {
            yield return new WaitForSeconds(delayAtaque);
            if (PlayerAreaAtaque)
            {

                PlayerController.Instance.caracteristica.LevarDano(valorDano, empurrarPlayer, _enemy._animacoes.Direcao);
                _animacao.atacar(_animacao.Direcao,() => {});
            }

            yield return new WaitForSeconds(frequenciaAtaque);
            iniciouProcessoAtaque = false;
        }

        public bool Atacando { get => _animacao.Atacando; }

    }

    [System.Serializable]
    public class MoveEnemy
    {
        public float Velocidade = 2;
        public float DistanciaPerseguirPlayer = 5f;
        public float DistanciaRecuo = 1;
        public float VelocidadeRecuo = 5;
        [HideInInspector]
        public Rigidbody2D _rb;

        EnemyController _enemy;
        Vector2 movimentoAnterior;

        Transform pontoRecuo;
        bool empurrando;

        public void Init(Rigidbody2D rb, EnemyController enemy)
        {
            this._rb = rb;
            _enemy = enemy;
            pontoRecuo = new GameObject("Ponto Recuo Inimigo: " + enemy.name).transform;
        }

        public void FixedUpdate()
        {
            /*
             * No caso do Enemy precisar ser empurrado
             * Significa que o Player bateu e pode empurrar o inimigo
            */
            if (empurrando)
            {
                if (Vector2.Distance(_enemy.transform.position, pontoRecuo.position) > 0.05f)
                {
                    _enemy.transform.position = Vector2.MoveTowards(
                        _enemy.transform.position,
                        pontoRecuo.position,
                        VelocidadeRecuo * Time.deltaTime
                    );
                }
                else
                {
                    empurrando = false;
                }
            }

            if (_enemy._animacoes.Atacando || _enemy._ataque.PlayerAreaAtaque || empurrando)
            {
                _enemy._animacoes.Caminhar(DirecaoEnemy.NONE);
                return;
            }

            /*
             * Caso o jogador chegue perto demais do monstro ele vai persegui-los
             * Andando usando o mesmo sistema do Player
            */
            if(Vector2.Distance(_enemy.transform.position, PlayerController.Instance.transform.position) <= DistanciaPerseguirPlayer)
            {
                //Debug.Log("Atras do Player");

                _enemy.transform.position = Vector2.MoveTowards(_enemy.transform.position, PlayerController.Instance.transform.position, Velocidade * Time.deltaTime);

                if(_enemy.transform.position.y > movimentoAnterior.y)
                {
                    //para Cima
                    _enemy._animacoes.Caminhar(DirecaoEnemy.UP);
                }
                else if(_enemy.transform.position.x > movimentoAnterior.x)
                {
                    //Para Direita
                    _enemy._animacoes.Caminhar(DirecaoEnemy.RIGHT);
                }
                else if(_enemy.transform.position.y < movimentoAnterior.y)
                {
                    //Para Baixo
                    _enemy._animacoes.Caminhar(DirecaoEnemy.DOWN);
                }
                else
                {
                    //para esquerda
                    _enemy._animacoes.Caminhar(DirecaoEnemy.LEFT);
                }

                movimentoAnterior = _enemy.transform.position;
            }
            else
            {
                //para esquerda
                _enemy._animacoes.Caminhar(DirecaoEnemy.NONE);
            }

        }

        public void Empurrar(DirecaoPlayer direcao)
        {
            //Debug.Log("Impulsinando Player, Direcao Playr: " + PlayerController.Instance.animacao.Direcao);
            Vector2 enemy = _enemy.transform.position;

            switch (direcao)
            {
                case DirecaoPlayer.UP:
                    pontoRecuo.position = new Vector2(enemy.x, enemy.y + DistanciaRecuo);
                    break;
                case DirecaoPlayer.RIGHT:
                    pontoRecuo.position = new Vector2(enemy.x + DistanciaRecuo, enemy.y);
                    break;
                case DirecaoPlayer.DOWN:
                    pontoRecuo.position = new Vector2(enemy.x, enemy.y - DistanciaRecuo);
                    break;
                case DirecaoPlayer.LEFT:
                    pontoRecuo.position = new Vector2(enemy.x - DistanciaRecuo, enemy.y);
                    break;
            }

            empurrando = true;
        }

    }

    [System.Serializable]
    public class CaractesristicaEnemy
    {
        public int Vida;
        public bool PodeSerEmpurrado;
        EnemyController _enemy;

        public void Init(EnemyController enemy)
        {
            _enemy = enemy;
        }

        public void LevarDano(int valor, DirecaoPlayer direcao)
        {
            Vida -= valor;

            if (PodeSerEmpurrado)
            {
                _enemy._move.Empurrar(direcao);
                _enemy._animacoes.TomarDano();
            }
            else
            {
                _enemy._animacoes.TomarDano();
            }

            Debug.Log("Vida do Inimigo "+ _enemy.name +" esta em: "+ Vida);

            if (Vida <= 0)
            {
                Debug.Log("Player Morto");

                _enemy._animacoes.Morrer(() => {

                    Debug.Log("Corpo do inimigo "+ _enemy.name + "Vai ficar no chão");

                });
            }

        }

        public bool Morto { get => Vida <= 0; }
    }

    public class EnemyController : MonoBehaviour
    {
        public AtaqueEnemy _ataque;
        public AnimacaoEnemy _animacoes;
        public MoveEnemy _move;
        public CaractesristicaEnemy _caractesristicas;
        public AudioEnemy _audio;

        void Start()
        {
            _animacoes.Init(GetComponent<Animator>(), transform.parent.GetComponent<Animator>(), this);
            _ataque.Init(_animacoes, this, GetComponentInChildren<EnemyAreaAtaque>());
            _move.Init(GetComponent<Rigidbody2D>(), this);
            _caractesristicas.Init(this);
            _audio.Init(GetComponent<AudioSource>());
        }

        void Update()
        {
            if (PlayerController.Instance.caracteristica.Morto || _caractesristicas.Morto)
                return;

            _ataque.Update();
        }

        void FixedUpdate()
        {
            if (PlayerController.Instance.caracteristica.Morto || _caractesristicas.Morto)
                return;

            _move.FixedUpdate();
        }

        public void IniciarCorotina(IEnumerator corotina)
        {
            StartCoroutine(corotina);
        }
    }
}
