﻿using System.Collections;
using System.Collections.Generic;
using Pippin.Enemy;
using UnityEngine;
using UnityEngine.UI;

namespace Pippin.Player
{

    public enum DirecaoPlayer
    {
        NONE, UP, RIGHT, DOWN, LEFT
    }

    public enum PescaPlayer
    {
        INIT, WHILE, FINISH
    }

    [System.Serializable]
    public class AudioPlayer
    {
        public AudioClip[] caminhada;
        public AudioClip[] ataque;
        public AudioClip[] levandoDano;
        public AudioClip[] morrendo;
        public AudioClip[] pescando;

        AudioSource Fonte;

        bool esperarAudio;

        public void Init(AudioSource fonte)
        {
            Fonte = fonte;
        }

        public void Caminhada()
        {
            if (!esperarAudio && caminhada.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = caminhada[Random.Range(0, caminhada.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        public void Ataque()
        {
            if (!esperarAudio && ataque.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = ataque[Random.Range(0, ataque.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        public void LevandoDano()
        {
            if (!esperarAudio && levandoDano.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = levandoDano[Random.Range(0, levandoDano.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        public void Morrendo()
        {
            if (!esperarAudio && morrendo.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = morrendo[Random.Range(0, morrendo.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        public void Pescando()
        {
            if (!esperarAudio && pescando.Length > 0)
            {
                esperarAudio = true;
                AudioClip escolhido = pescando[Random.Range(0, pescando.Length - 1)];
                Fonte.clip = escolhido;
                Fonte.Stop();
                Fonte.Play();
                EsperarAudioFinalizar(escolhido.length, () => { esperarAudio = false; });
            }
        }

        IEnumerator EsperarAudioFinalizar(float time, System.Action callback)
        {
            yield return new WaitForSeconds(time);
            callback();
        }

    }

    [System.Serializable]
    public class AnimacaoPlayer
    {
        [Header("Clips de Animacoes")]
        public AnimationClip Ataque;
        public AnimationClip Caminhada;
        public AnimationClip damage;
        public AnimationClip morte;
        public AnimationClip startPeca;

        [Header("Ajustes de velocidade Animacoes")]
        public float multiplicadorWalk = 1f;
        public float multiplicadorIdle = 1f;
        public float multiplicadorAtaque = 1f;

        Animator _animador;
        Animator _parent;
        int ultimaDirecao = 1;

        public void Init(Animator animador, Animator parent)
        {
            _animador = animador;
            _parent = parent;
            _animador.SetFloat("mul-walk", multiplicadorWalk);
            _animador.SetFloat("mul-ataque", multiplicadorAtaque);
            _animador.SetFloat("mul-idle", multiplicadorIdle);
        }

        public void Morrer(System.Action esperarAnimacao = null)
        {
            ResetarParametrosAnimator();
            switch (Direcao)
            {
                case DirecaoPlayer.UP:

                    _animador.SetInteger("morto", 1);
                    if (esperarAnimacao != null)
                        PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
                case DirecaoPlayer.RIGHT:

                    _animador.SetInteger("morto", 2);
                    if (esperarAnimacao != null)
                        PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
                case DirecaoPlayer.DOWN:

                    _animador.SetInteger("morto", 3);
                    if (esperarAnimacao != null)
                        PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
                case DirecaoPlayer.LEFT:

                    _animador.SetInteger("morto", 4);
                    if (esperarAnimacao != null)
                        PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(morte.length, esperarAnimacao));

                    break;
            }

            PlayerController.Instance._audio.Morrendo();
        }

        public void Pescar(PescaPlayer pesca, float time = 0, System.Action callback = null)
        {
            switch (pesca)
            {
                case PescaPlayer.INIT:
                    _animador.SetInteger("fish-start", ultimaDirecao);
                    PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(time, () =>
                    {
                        _animador.SetInteger("fish-start", 0);
                        if (callback != null)
                            callback();
                    }));

                    break;

                case PescaPlayer.WHILE:
                    _animador.SetTrigger("fish-atencao");
                    PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(time, () =>
                    {
                        if (callback != null)
                            callback();
                    }));

                    break;

                case PescaPlayer.FINISH:
                    _animador.SetTrigger("fish-get");
                    PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(time, () =>
                    {
                        if (callback != null)
                            callback();
                    }));
                    break;
            }

            PlayerController.Instance._audio.Pescando();
        }

        public void TomarDano(System.Action esperarAnimacao = null)
        {
            _parent.SetTrigger("dano");
            if (esperarAnimacao != null)
                PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(damage.length, esperarAnimacao));

            PlayerController.Instance._audio.LevandoDano();
        }

        public void Caminhar(DirecaoPlayer caminhada)
        {
            switch (caminhada)
            {
                case DirecaoPlayer.UP:
                    _animador.SetInteger("caminhar", 1);
                    ultimaDirecao = 1;
                    break;
                case DirecaoPlayer.RIGHT:
                    _animador.SetInteger("caminhar", 2);
                    ultimaDirecao = 2;
                    break;
                case DirecaoPlayer.DOWN:
                    _animador.SetInteger("caminhar", 3);
                    ultimaDirecao = 3;
                    break;
                case DirecaoPlayer.LEFT:
                    _animador.SetInteger("caminhar", 4);
                    ultimaDirecao = 4;
                    break;
                default:
                    _animador.SetInteger("caminhar", 0);
                    break;
            }

            PlayerController.Instance._audio.Caminhada();
        }

        public void atacar(DirecaoPlayer direcao, System.Action esperarAtaqueFinalizar)
        {
            switch (direcao)
            {
                case DirecaoPlayer.UP:

                    _animador.SetInteger("atacar", 1);
                    PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () =>
                    {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
                case DirecaoPlayer.RIGHT:

                    _animador.SetInteger("atacar", 2);
                    PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () =>
                    {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
                case DirecaoPlayer.DOWN:

                    _animador.SetInteger("atacar", 3);
                    PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () =>
                    {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
                case DirecaoPlayer.LEFT:

                    _animador.SetInteger("atacar", 4);
                    PlayerController.Instance.IniciarCorotina(esperarAcaoAnimacao(Ataque.length, () =>
                    {
                        _animador.SetInteger("atacar", 0);
                        esperarAtaqueFinalizar();
                    }));

                    break;
            }
            PlayerController.Instance._audio.Ataque();
        }

        public void ResetarParametrosAnimator()
        {
            _animador.SetInteger("caminhar", 0);
            _animador.SetInteger("atacar", 0);
            _animador.SetInteger("start-fish", 0);
            _animador.Rebind();
        }

        IEnumerator esperarAcaoAnimacao(float time, System.Action callback)
        {
            yield return new WaitForSeconds(time);
            callback();
        }

        #region metodos_get_set

        public bool Atacando { get => _animador.GetInteger("atacar") != 0; }

        public DirecaoPlayer Direcao { get => (DirecaoPlayer)ultimaDirecao; }

        #endregion
    }

    [System.Serializable]
    public class PescarPlayer
    {
        public int minTempoPescando = 5;
        public int maxTempoPescando = 10;

        public float tempoEmAtencao = 1;

        //[HideInInspector]
        public bool AreaPesca;
        //[HideInInspector]
        public Transform posicaoPesca;
        //[HideInInspector]
        public bool pescando;
        bool fisgarPeixe;
        //[HideInInspector]
        public bool Cancelado;
        bool habilitarPegarPeixe;

        public void Init()
        {

        }

        public void Update()
        {
            if (PlayerController.Instance.caracteristica.Morto || Cancelado)
                return;

            if(AreaPesca && !pescando && Input.GetKeyDown(KeyCode.Space))
            {
                pescando = true;
                IniciarPesca();
            }

            if (Input.GetKeyDown(KeyCode.Space) && habilitarPegarPeixe)
            {
                fisgarPeixe = true;
            }

            if (Input.GetKeyDown(KeyCode.P))
            {
                Cancelado = true;
                CancelarPesca();
            }

        }

        void IniciarPesca()
        {
            PlayerController.Instance.transform.position = posicaoPesca.position;
            float time = Random.Range(minTempoPescando, maxTempoPescando);
            PlayerController.Instance.animacao.Pescar(PescaPlayer.INIT, time, AtencaoPesca);
        }

        void AtencaoPesca()
        {
            PlayerController.Instance.StartCoroutine(EsperarTempoAtencao());
        }

        IEnumerator EsperarTempoAtencao()
        {
            habilitarPegarPeixe = true;
            yield return new WaitForSeconds(tempoEmAtencao);
            PlayerController.Instance.animacao.Pescar(
                PescaPlayer.WHILE,
                tempoEmAtencao,
                () => {
                    if (!fisgarPeixe)
                    {
                        IniciarPesca();
                        fisgarPeixe = false;
                        habilitarPegarPeixe = false;
                    }
                    else
                    {
                        PegarPeixe();
                        fisgarPeixe = false;
                        habilitarPegarPeixe = false;
                    }
                }
            );
        }

        void PegarPeixe()
        {
            PlayerController.Instance.animacao.Pescar(
                PescaPlayer.FINISH,2, FinalizarPesca
            );
        }

        void FinalizarPesca()
        {
            pescando = false;
        }

        public void CancelarPesca()
        {
            pescando = false;
            PlayerController.Instance.StopAllCoroutines();
            PlayerController.Instance.animacao.ResetarParametrosAnimator();
        }

    }

    [System.Serializable]
    public class AtaquePlayer
    {
        public int valorDano;

        //public delegate void InimigoAtacado(int valorDano, System.Action callback = null);
        //public List<InimigoAtacado> inimigoAtacado;
        public List<EnemyController> _enemys; 

        public void Init()
        {
            _enemys = new List<EnemyController>();
        }

        public void Update()
        {
            if (PlayerController.Instance.animacao.Atacando)
                return;

            if (Input.GetKeyDown(KeyCode.J) || Input.GetKeyDown(KeyCode.K))
            {
                PlayerController.Instance.animacao.atacar(PlayerController.Instance.animacao.Direcao, () => { });
                for (int i = 0; i < _enemys.Count; i++)
                {
                    _enemys[i]._caractesristicas.LevarDano(valorDano, PlayerController.Instance.animacao.Direcao);
                }
                _enemys.Clear();
            }
        }

        public bool Atacando { get => PlayerController.Instance.animacao.Atacando; }

    }

    [System.Serializable]
    public class MovePlayer
    {
        [Header("Velocidade de caminhada do Player")]
        public float Velocidade;

        [Header("Configuracao de Empurro do Player por Parte dos Inimigos")]
        public float DistanciaRecuo = 1;
        public float VelocidadeRecuo = 5;

        [HideInInspector]
        public Rigidbody2D _rb;
        Transform pontoRecuo;
        bool empurrando;

        public void Init(Rigidbody2D rb)
        {
            this._rb = rb;
            pontoRecuo = new GameObject("Recuo Player").transform;
        }

        public void FixedUpdate()
        {
            /*
             * No caso do Player precisar ser empurrado
             * Significa que um inimigo forte bateu no Player
            */
            if(empurrando)
            {
                if(Vector2.Distance(PlayerController.Instance.transform.position, pontoRecuo.position) > 0.05f)
                {
                    PlayerController.Instance.transform.position = Vector2.MoveTowards(
                        PlayerController.Instance.transform.position,
                        pontoRecuo.position,
                        VelocidadeRecuo * Time.deltaTime
                    );
                }
                else
                {
                    empurrando = false;
                }
            }

            /*
             * Verificando se o Player esta atacando
             * ou esta sendo empurrado pelo inimigo
             * Aqui o player não pode caminhar
            */
            if (PlayerController.Instance.animacao.Atacando || empurrando || PlayerController.Instance.pesca.pescando)
            {
                _rb.velocity = new Vector2(0, 0);
                PlayerController.Instance.animacao.Caminhar(DirecaoPlayer.NONE);
                return;
            }

            /*
             * Verificação de direção que o player deseja andar
            */
            if (Input.GetKey(KeyCode.UpArrow) || Input.GetKey(KeyCode.W))
            {
                //para Cima
                _rb.velocity = new Vector2(0, Velocidade);
                PlayerController.Instance.animacao.Caminhar(DirecaoPlayer.UP);
            }
            else if (Input.GetKey(KeyCode.DownArrow) || Input.GetKey(KeyCode.S))
            {
                //para baixo
                _rb.velocity = new Vector2(0, -Velocidade);
                PlayerController.Instance.animacao.Caminhar(DirecaoPlayer.DOWN);
            }
            else if (Input.GetKey(KeyCode.LeftArrow) || Input.GetKey(KeyCode.A))
            {
                //para esquerda
                _rb.velocity = new Vector2(-Velocidade, 0);
                PlayerController.Instance.animacao.Caminhar(DirecaoPlayer.LEFT);
            }
            else if (Input.GetKey(KeyCode.RightArrow) || Input.GetKey(KeyCode.D))
            {
                //para Direita
                _rb.velocity = new Vector2(Velocidade, 0);
                PlayerController.Instance.animacao.Caminhar(DirecaoPlayer.RIGHT);
            }
            else
            {
                _rb.velocity = new Vector2(0, 0);
                PlayerController.Instance.animacao.Caminhar(DirecaoPlayer.NONE);
            }

        }

        public void Empurrar(DirecaoEnemy direcao)
        {
            //Debug.Log("Impulsinando Player, Direcao Playr: " + PlayerController.Instance.animacao.Direcao);
            Vector2 player = PlayerController.Instance.transform.position;

            switch (direcao)
            {
                case DirecaoEnemy.UP:
                    pontoRecuo.position = new Vector2(player.x, player.y + DistanciaRecuo);
                    break;
                case DirecaoEnemy.RIGHT:
                    pontoRecuo.position = new Vector2(player.x + DistanciaRecuo, player.y);
                    break;
                case DirecaoEnemy.DOWN:
                    pontoRecuo.position = new Vector2(player.x, player.y - DistanciaRecuo);
                    break;
                case DirecaoEnemy.LEFT:
                    pontoRecuo.position = new Vector2(player.x - DistanciaRecuo, player.y);
                    break;
            }

            empurrando = true;
        }

        public void resetarVelocity()
        {
            _rb.velocity = new Vector2(0, 0);
            PlayerController.Instance.animacao.Caminhar(DirecaoPlayer.NONE);
        }
    }

    [System.Serializable]
    public class CaractesristicaPlayer
    {
        public int Vida;
        public Slider VidaSlider;


        public void Init()
        {
            VidaSlider.maxValue = Vida;
            VidaSlider.value = Vida;
        }

        public void LevarDano(int valor, bool empurrar, DirecaoEnemy direcao)
        {
            Vida -= valor;
            VidaSlider.value = Vida;

            if (empurrar)
            {
                PlayerController.Instance.move.Empurrar(direcao);
                PlayerController.Instance.animacao.TomarDano();
            }
            else
            {
                PlayerController.Instance.animacao.TomarDano();
            }

            Debug.Log("Vida do player "+ Vida);

            if(Vida <= 0)
            {
                Debug.Log("Player Morto");

                PlayerController.Instance.animacao.Morrer(() => {

                    Debug.Log("Reiniciar Scene");

                });
            }

        }

        public bool Morto { get => Vida <= 0; }
    }

    public class PlayerController : MonoBehaviour
    {
        public static PlayerController Instance;

        public MovePlayer move;
        public AnimacaoPlayer animacao;
        public AtaquePlayer ataque;
        public CaractesristicaPlayer caracteristica;
        public PescarPlayer pesca;
        public AudioPlayer _audio;

        [HideInInspector]
        public bool esperarPassagemSala;

        private void Awake()
        {
            Instance = this;
        }

        void Start()
        {
            move.Init(GetComponent<Rigidbody2D>());
            animacao.Init(GetComponent<Animator>(), transform.parent.GetComponent<Animator>());
            ataque.Init();
            pesca.Init();
            _audio.Init(GetComponent<AudioSource>());
            caracteristica.Init();
        }

        void Update()
        {
            if(caracteristica.Morto || esperarPassagemSala)
            {

                return;
            }

            if(Input.GetKeyDown(KeyCode.B))
            {
                animacao.TomarDano();
            }

            ataque.Update();
            pesca.Update();
        }

        void FixedUpdate()
        {
            if (caracteristica.Morto || esperarPassagemSala)
            {
                move.resetarVelocity();
                return;
            }

            move.FixedUpdate();
        }

        public void IniciarCorotina(IEnumerator corotina)
        {
            StartCoroutine(corotina);
        }
    }
}
